import React, { useState, useEffect } from "react";
import { useModelPhoto } from "@/hooks/modelPhoto";
import Button from "@/components/UI/button/Button";
import InputField from "@/components/UI/input/InputField";
import SelectInput, { Option } from "@/components/UI/input/SelectInput";
import CircleLoader from "@/components/UI/loader/circle/CircleLoader";
const AddModelPhoto = () => {
  //use model photo hook
  const { addModelPhoto, fetchAllModelPhotos } = useModelPhoto();
  //all models
  const [allModelPhotos, setAllModelPhotos] = useState([]);
  //set loading state
  const [loading, setLoading] = useState(false);
  //set image preview url
  const [url, setUrl] = useState(null);
  //set all input fields
  const [inputs, setInputs] = useState({
    image_url: null,
    model_id: null,
    orientation: "",
  });
  // InputValidationErrors
  const [validationErrors, setValidationErrors] = useState({});
  //define errors
  const [errors, setErrors] = useState([]);
  //handleInputChange
  const handleInputChange = (e) => {
    setInputs({ ...inputs, [e.target.name]: e.target.value });
  };

  //handle uploadFileChange
  const handleUploadFileChange = (e) => {
    // Get the selected file (only one file)
    const file = e.target.files[0];
    // this gives us the data on what files are selected
    if (file) {
      console.log("[file]", file);
      // transform files into data urls
      const url = URL.createObjectURL(file);
      //set image preview url
      setUrl(url);
      //set inputs data
      setInputs({ ...inputs, [e.target.name]: file });
    }
  };

  //check input validation
  const validate = (inputValues) => {
    const errors = {};

    if (!inputValues.image_url) {
      errors.image_url = "Model Image is required";
    }
    if (!inputValues.orientation) {
      errors.orientation = "Model Picture Orientation is required";
    }
    if (!inputValues.model_id) {
      errors.model_id = "Model ID is required";
    }
    return errors;
  };

  //check validation error object
  const isObjectEmpty = (objectName) => {
    return (
      objectName &&
      Object.keys(objectName).length === 0 &&
      objectName.constructor === Object
    );
  };

  //submit the data in backend
  const handleSubmit = (event) => {
    //event is an event object
    event.preventDefault();
    //client-side validation
    const errors = validate(inputs);
    //set validation error in state object
    setValidationErrors(errors);
    if (!isObjectEmpty(errors)) {
      //if any errors occur return null and do not submit the input values
      return;
    } else {
      //extract inputs
      const { image_url, model_id, orientation } = inputs;

      // here you calling your submit api
      addModelPhoto({
        setLoading,
        setErrors,
        setInputs,
        image_url,
        model_id,
        orientation,
      });

      //dispatch action when you use redux

      //reset form
      setInputs({
        image: null,
        model_id: null,
        orientation: "",
      });
    }
  };

  useEffect(() => {
    fetchAllModelPhotos(setAllModelPhotos);
  }, []);

  return (
    <>
      {loading ? (
        <CircleLoader />
      ) : (
        <div className="w-full bg-white p-4 flex flex-col justify-center items-center rounded-md mt-10 gap-5">
          <h2 className="font-textImpact text-center">ADD MODEL</h2>
          <form
            className="w-[85%] flex flex-col justify-center items-center gap-3"
            onSubmit={handleSubmit}
          >
            <div className="w-11/12 grid grid-cols-12 justify-center items-center">
              <div className="col-span-6">
                <SelectInput
                  name="model_id"
                  onChange={handleInputChange}
                  className="w-11/12 bg-gray-200 rounded-full p-3 placeholder:text-black placeholder:font-title placeholder:font-semibold drop-shadow-md"
                  errorMessage={validationErrors["model_id"]}
                >
                  {allModelPhotos &&
                    allModelPhotos.length > 0 &&
                    allModelPhotos.map((model) => (
                      <Option option={model.name} value={model.id} />
                    ))}
                </SelectInput>
              </div>
              <div className="col-span-6">
                <SelectInput
                  name="orientation"
                  onChange={handleInputChange}
                  className="w-11/12 bg-gray-200 rounded-full p-3 placeholder:text-black placeholder:font-title placeholder:font-semibold drop-shadow-md"
                  errorMessage={validationErrors["orientation"]}
                >
                  <Option option="Vertical" value="vertical" />
                  <Option option="Horizontal" value="horizontal" />
                </SelectInput>
              </div>
            </div>
            <div className="w-11/12 grid grid-cols-12 justify-center items-center">
              <div className="col-span-6">
                <InputField
                  type="file"
                  name="image_url"
                  className="w-11/12 bg-gray-200 rounded-full p-3 placeholder:text-black placeholder:font-title placeholder:font-semibold drop-shadow-md"
                  onChange={handleUploadFileChange}
                  errorMessage={validationErrors["image_url"]}
                />
              </div>
              <div className="col-span-6">
                <div className="relative w-[85%] h-[300px] m-5">
                  <img
                    src={url}
                    className="absolute w-full h-full object-contain"
                  />
                </div>
              </div>
            </div>
            <Button type="submit" text="ADD MODEL PHOTO" />
          </form>
        </div>
      )}
    </>
  );
};

export default AddModelPhoto;
